#!/usr/bin/env python3
# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
#
# Adapted for Debian to not fail on missing forex_python.
import gettext
import os
import sys
from argparse import ArgumentParser

import pycountry
try:
    from forex_python.converter import CurrencyCodes
except ImportError:
    CurrencyCodes = None

try:
    from progressbar import ProgressBar, Bar, ETA, SimpleProgress
except ImportError:
    ProgressBar = None

try:
    from proteus import Model, config
except ImportError:
    prog = os.path.basename(sys.argv[0])
    sys.exit("proteus must be installed to use %s" % prog)

symbols = {
    'AED': 'د.إ;',
    'AFN': 'Afs',
    'ALL': 'L',
    'AMD': 'AMD',
    'ANG': 'NAƒ',
    'AOA': 'Kz',
    'ARS': '$',
    'AUD': '$',
    'AWG': 'ƒ',
    'AZN': 'AZN',
    'BAM': 'KM',
    'BBD': 'Bds$',
    'BDT': '৳',
    'BGN': 'BGN',
    'BHD': '.د.ب',
    'BIF': 'FBu',
    'BMD': 'BD$',
    'BND': 'B$',
    'BOB': 'Bs.',
    'BRL': 'R$',
    'BSD': 'B$',
    'BTN': 'Nu.',
    'BWP': 'P',
    'BZD': 'BZ$',
    'CAD': '$',
    'CDF': 'F',
    'CHF': 'Fr.',
    'CLP': '$',
    'CNY': '¥',
    'COP': 'Col$',
    'CRC': '₡',
    'CUC': '$',
    'CVE': 'Esc',
    'CZK': 'Kč',
    'DJF': 'Fdj',
    'DKK': 'Kr',
    'DOP': 'RD$',
    'DZD': 'د.ج',
    'EGP': '£',
    'ERN': 'Nfa',
    'ETB': 'Br',
    'EUR': '€',
    'FJD': 'FJ$',
    'FKP': '£',
    'GBP': '£',
    'GEL': 'GEL',
    'GHS': 'GH₵',
    'GIP': '£',
    'GMD': 'D',
    'GNF': 'FG',
    'GTQ': 'Q',
    'GYD': 'GY$',
    'HKD': 'HK$',
    'HNL': 'L',
    'HRK': 'kn',
    'HTG': 'G',
    'HUF': 'Ft',
    'IDR': 'Rp',
    'ILS': '₪',
    'INR': '₹',
    'IQD': 'د.ع',
    'IRR': 'IRR',
    'ISK': 'kr',
    'JMD': 'J$',
    'JOD': 'JOD',
    'JPY': '¥',
    'KES': 'KSh',
    'KGS': 'сом',
    'KHR': '៛',
    'KMF': 'KMF',
    'KPW': 'W',
    'KRW': 'W',
    'KWD': 'KWD',
    'KYD': 'KY$',
    'KZT': 'T',
    'LAK': 'KN',
    'LBP': '£',
    'LKR': 'Rs',
    'LRD': 'L$',
    'LSL': 'M',
    'LYD': 'LD',
    'MAD': 'MAD',
    'MDL': 'MDL',
    'MGA': 'FMG',
    'MKD': 'MKD',
    'MMK': 'K',
    'MNT': '₮',
    'MOP': 'P',
    'MRO': 'UM',
    'MUR': 'Rs',
    'MVR': 'Rf',
    'MWK': 'MK',
    'MXN': '$',
    'MYR': 'RM',
    'NAD': 'N$',
    'NGN': '₦',
    'NIO': 'C$',
    'NOK': 'kr',
    'NPR': 'NRs',
    'NZD': 'NZ$',
    'OMR': 'OMR',
    'PAB': 'B./',
    'PEN': 'S/.',
    'PGK': 'K',
    'PHP': '₱',
    'PKR': 'Rs.',
    'PLN': 'zł',
    'PYG': '₲',
    'QAR': 'QR',
    'RON': 'L',
    'RSD': 'din.',
    'RUB': 'R',
    'SAR': 'SR',
    'SBD': 'SI$',
    'SCR': 'SR',
    'SDG': 'SDG',
    'SEK': 'kr',
    'SGD': 'S$',
    'SHP': '£',
    'SLL': 'Le',
    'SOS': 'Sh.',
    'SRD': '$',
    'SYP': 'LS',
    'SZL': 'E',
    'THB': '฿',
    'TJS': 'TJS',
    'TMT': 'm',
    'TND': 'DT',
    'TRY': 'TRY',
    'TTD': 'TT$',
    'TWD': 'NT$',
    'TZS': 'TZS',
    'UAH': 'UAH',
    'UGX': 'USh',
    'USD': 'US$',
    'UYU': '$U',
    'UZS': 'UZS',
    'VND': '₫',
    'VUV': 'VT',
    'WST': 'WS$',
    'XAF': 'CFA',
    'XCD': 'EC$',
    'XDR': 'SDR',
    'XOF': 'CFA',
    'XPF': 'F',
    'YER': 'YER',
    'ZAR': 'R',
}


def _progress(iterable):
    if ProgressBar:
        pbar = ProgressBar(
            widgets=[SimpleProgress(), Bar(), ETA()])
    else:
        pbar = iter
    return pbar(iterable)


def _get_language_codes():
    Language = Model.get('ir.lang')
    languages = Language.find([('translatable', '=', True)])
    for l in languages:
        yield l.code


def _remove_forbidden_chars(name):
    from trytond.model.fields import Char
    for c in Char.forbidden_chars:
        if c in name:
            name.replace(c, ' ')
    return name.strip()


def get_currencies():
    Currency = Model.get('currency.currency')
    return {c.code: c for c in Currency.find([])}


def update_currencies(currencies):
    print("Update currencies", file=sys.stderr)
    Currency = Model.get('currency.currency')
    if CurrencyCodes:
        codes = CurrencyCodes()
    else:
        codes = None

    records = []
    for currency in _progress(pycountry.currencies):
        code = currency.alpha_3
        if code in currencies:
            record = currencies[code]
        else:
            record = Currency(code=code)
        record.name = _remove_forbidden_chars(currency.name)
        record.numeric_code = currency.numeric
        if codes:
            record.symbol = (codes.get_symbol(currency.alpha_3)
                or currency.alpha_3)
        else:
            record.symbol = symbols.get(currency.alpha_3) or currency.alpha_3
        records.append(record)

    Currency.save(records)
    return {c.code: c for c in records}


def translate_currencies(currencies):
    Currency = Model.get('currency.currency')

    current_config = config.get_config()
    for code in _get_language_codes():
        try:
            gnutranslation = gettext.translation(
                'iso4217', pycountry.LOCALES_DIR, languages=[code])
        except IOError:
            continue
        print("Update currencies %s" % code, file=sys.stderr)
        with current_config.set_context(language=code):
            records = []
            for currency in _progress(pycountry.currencies):
                record = Currency(currencies[currency.alpha_3].id)
                record.name = _remove_forbidden_chars(
                    gnutranslation.gettext(currency.name))
                records.append(record)
            Currency.save(records)


def main(database, config_file=None):
    config.set_trytond(database, config_file=config_file)
    with config.get_config().set_context(active_test=False):
        do_import()


def do_import():
    currencies = get_currencies()
    currencies = update_currencies(currencies)
    translate_currencies(currencies)


def run():
    parser = ArgumentParser()
    parser.add_argument('-d', '--database', dest='database', required=True)
    parser.add_argument('-c', '--config', dest='config_file',
        help='the trytond config file')

    args = parser.parse_args()
    main(args.database, args.config_file)


if __name__ == '__main__':
    run()
